@@ -68,7 +68,10 @@ print.nb <- function(x, ...) {
 }
 
 summary.listw <- function(object, coords=NULL, longlat=FALSE, 
-	zero.policy=FALSE, scale=1, ...) {
+	zero.policy=NULL, scale=1, ...) {
+        if (is.null(zero.policy))
+            zero.policy <- get("zeroPolicy", env = .spdepOptions)
+        stopifnot(is.logical(zero.policy))
 	cat("Characteristics of weights list object:\n")
 	summary(object$neighbours, coords=coords, longlat=longlat, 
 		scale=scale, ...)
@@ -81,7 +84,10 @@ summary.listw <- function(object, coords=NULL, longlat=FALSE,
 
 }
 
-print.listw <- function(x, zero.policy=FALSE, ...) {
+print.listw <- function(x, zero.policy=NULL, ...) {
+        if (is.null(zero.policy))
+            zero.policy <- get("zeroPolicy", env = .spdepOptions)
+        stopifnot(is.logical(zero.policy))
 	cat("Characteristics of weights list object:\n")
 	print.nb(x$neighbours, ...)
 	style <- x$style
